
/*
 * $Id: GoalRec.cc,v 1.4 1999/09/07 17:46:10 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "ILog.h"
#include "OLog.h"
#include "GoalRec.h"
#include "gadgets.h"


template <class T>
inline
void SetOrInc(T &var, const T &inc) {
	if (var < 0)
		var = inc;
	else
	if (inc >= 0)
		var += inc;
}


/* ErrGoalRec */

ErrGoalRec::ErrGoalRec(): theRatio(-1), theCount(-1) {
}

ErrGoalRec &ErrGoalRec::operator +=(const ErrGoalRec &e) {

	SetOrInc(theCount, e.theCount);

	if (theRatio < 0)
		theRatio = e.theRatio;
	else
	if (e.theRatio >= 0)
		theRatio = Min(theRatio, e.theRatio);

	return *this;
}

OLog &ErrGoalRec::store(OLog &ol) const {
	return ol << theRatio << theCount;
}

ILog &ErrGoalRec::load(ILog &il) {
	return il >> theRatio >> theCount;
}

ostream &ErrGoalRec::print(ostream &os) const {
	if (theRatio > 0)
		return os << 100*theRatio << '%';
	if (theCount > 0)
		return os << theCount;
	return os << "<none>";
}


/* GoalRec */

GoalRec::GoalRec(): theXactCnt(-1), theFillSz(-1) {
}

GoalRec::operator void*() const {
	// note: errors alone do not count as a real goal
	return theDuration >= 0 || theXactCnt >= 0 || theFillSz >= 0 ?
		(void*)-1 : 0;
}

GoalRec &GoalRec::operator +=(const GoalRec &g) {
	// duration is not updated unless it was not set before
	if (theDuration < 0)
		theDuration = g.theDuration;
	else
		Assert(g.theDuration >= 0);

	SetOrInc(theXactCnt, g.theXactCnt);
	SetOrInc(theFillSz, g.theFillSz);

	theErrs += g.theErrs;

	return *this;
}

OLog &GoalRec::store(OLog &ol) const {
	return ol << theDuration << theXactCnt << theFillSz << theErrs;
}

ILog &GoalRec::load(ILog &il) {
	return il >> theDuration >> theXactCnt >> theFillSz >> theErrs;
}

ostream &GoalRec::print(ostream &os) const {
#define grp_stream_hack  (count++ ? os << ':' : os)
	int count = 0;

	if (theDuration >= 0)
		grp_stream_hack << theDuration;
	if (theXactCnt >= 0)
		grp_stream_hack << theXactCnt;
	if (theFillSz >= 0)
		grp_stream_hack << theFillSz;
	if (theErrs)
		theErrs.print(grp_stream_hack);

	if (!count)
		os << "<none>";

	return os;
}
