
/*
 * $Id: HRStat.cc,v 1.3 1999/09/07 17:46:10 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "ILog.h"
#include "OLog.h"
#include "HRStat.h"
#include "gadgets.h"


HRStat::HRStat() {
	reset();
}

void HRStat::reset() {
	theHits.reset();
	theMisses.reset();
}

OLog &HRStat::store(OLog &log) const {
	return log << theHits << theMisses;
}

ILog &HRStat::load(ILog &log) {
	return log >> theHits >> theMisses;
}

double HRStat::dhr() const {
	return Ratio(hits(), xacts());
}

double HRStat::bhr() const {
	const BigSize denom = theHits.size() + theMisses.size();
	return denom != 0 ? theHits.size() / denom : -1;
}

double HRStat::dhp() const {
	return Percent(hits(), xacts());
}

double HRStat::bhp() const {
	const BigSize denom = theHits.size() + theMisses.size();
	return denom != 0 ? 100. * (theHits.size() / denom) : -1;
}

HRStat &HRStat::operator +=(const HRStat &hr) {
	theHits += hr.theHits;
	theMisses += hr.theMisses;
	return *this;
}
