
/*
 * $Id: IntvlCut.cc,v 1.3 1999/09/07 17:46:13 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "IntvlCut.h"


IntervalCutter::IntervalCutter() {
}

void IntervalCutter::configure(Time anIntvl) {
	Assert(anIntvl != 0);
	theIntvl = anIntvl;
}

bool IntervalCutter::update(Time curTime) {
	Assert(curTime >= 0);

	// first call, just set time
	if (theEoi < 0) {
		theEoi = curTime;
		return false;
	}

	// if the cutter is disabled return true 
	if (theIntvl < 0) {
		theEoi = curTime;
		return true;
	}

	// check for eoi
	if (curTime > theEoi + theIntvl) {
		// note: we use (a,b] interval to ensure that
		// if two events happen at the same time, they
		// always end up in the same interval!
		do {
			theEoi += theIntvl;
		} while (curTime > theEoi + theIntvl);
		return true;
	}

	return false;
}
