
/*
 * $Id: PolySrv.cc,v 1.4 1999/09/07 17:46:22 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include <iostream.h>
#include <iomanip.h>

#include "Clock.h"
#include "Server.h"
#include "StatPhaseMgr.h"
#include "Rnd.h"
#include "OLog.h"
#include "LogComment.h"
#include "SharedOpts.h"
#include "SrvOpts.h"
#include "ServerSym.h"
#include "PolySrv.h"

#include "gadgets.h"
#include "shutdown.h"
#include "globals.h"


bool PolySrv::findPort(const ServerSym *s) {
	const int port = s->host().port();
	Assert(port >= 0);
	for (int i = 0; i < theLocalCfgs.count(); ++i) {
		if (theLocalCfgs[i] != s && theLocalCfgs[i]->host().port() == port)
			return true;
	}
	return false;
}

Agent *PolySrv::makeAgent(const AgentSym *c) {
	Assert(c);

	if (!c->isA("Server"))
		return 0;
	
	const ServerSym *cfg = &(const ServerSym&)c->cast("Server");
	const bool bind2iface = cfg->host().port() >= 0 && findPort(cfg);
	Server *server = new Server();
	server->configure(cfg, bind2iface);
	
	return server;
}

void PolySrv::configure() {
	PolyApp::configure();
}

void PolySrv::getOpts(Array<OptGrp*> &opts) {
	PolyApp::getOpts(opts);
	opts.append(&TheSrvOpts);
}


int main(int argc, char *argv[]) {
	PolySrv app;
	return app.run(argc, argv);
}
