
/*
 * $Id: SharedOpts.cc,v 1.17 1999/09/17 23:54:19 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "Select.h"
#include "Poll.h"
#include "SharedOpts.h"
#include "globals.h"

SharedOpts TheOpts;

SharedOpts::SharedOpts():
	theHelpOpt(this,      "help",               "list of options"),
	theVersOpt(this,      "version",            "package version info"),
	theHostTypeOpt(this,  "host_type",          "package build environment info"),

	theVerbLevel(this,    "verb_lvl <int>",     "console verbosity level", 0),
	printReqs(this,       "prn_reqs <bool>",    "dump request headers to console"),
	printReps(this,       "prn_reps <bool>",    "dump reply headers to console"),
	printErrs(this,       "prn_errs <bool>",    "dump msg headers to console on some xaction errors"),
	thePrnDumpSize(this,  "prn_dump_size <size>","limit size of individual msg dump", Size::KB(1)),
	theNotifAddr(this,    "notify <addr>",      "send notifications to a [remote] monitoring program", NetAddr()),
	theRunLabel(this,     "label <str>",        "run label"),

	theFDLimit(this,      "fd_limit <int>",     "decrease default fd limit"),

	theCfgFileName(this,  "config <file>",		"configuration file name"),
	theCfgDirs(this,      "cfg_dirs <list>",    "directories to search for cfg files"),

	theConsFileName(this, "console <file>",     "redirect console output", "-"),
	theLogFileName(this,  "log <file>",         "detailed binary log file"),
	theLogSize(this,      "log_size <size>",    "binary log size limit"),
	theStatCycleLen(this, "stats_cycle <time>", "statistics cycle length", Time::Sec(5)),

	theFileScanner(this,  "file_scan <poll|select>", "system call to use for scanning ready files", new PG_PREFFERED_FILE_SCANNER),
	theFakeHosts(this,    "fake_hosts <list>",  "use given addresses, do not lookup real interfaces"),

	theIdleTout(this,	  "idle_tout <time>",   "exit after inactivity timeout"),

	theRngSeed(this,      "rng_seed <int>",     "general purpose r.n.g. seed", 1)
{
}

bool SharedOpts::validate() const {
	if (theRngSeed <= 0)
		cerr << ThePrgName << ": r.n.g. seed must be positive; got: " << theRngSeed << endl;
	else
	if (theStatCycleLen <= 0)
		cerr << ThePrgName << ": the stats cycle length must be positive; got: " << theStatCycleLen << endl;
	else
	if (!theFileScanner)
		cerr << ThePrgName << ": the file scanner (e.g. `select' or `poll') must be specified" << endl;
	else
	if (!theCfgFileName)
		cerr << ThePrgName << ": configuration file must be specified (see --config option)" << endl;
	else
		return OptGrp::validate();
	return false;
}
