
/*
 * $Id: Client.h,v 1.19 1999/09/17 23:54:33 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_CLIENT_CLASS_H
#define PG_CLIENT_CLASS_H

#include "AlarmClock.h"
#include "Farm.h"
#include "Agent.h"
#include "CltXact.h"
#include "BcastRcver.h"

class SockOpt;
class Connection;
class RndDistr;
class OidRec;
class CltConnMgr;
class ServerRep;
class RobotSym;
class PopModel;

class Client: public Agent, public AlarmUser, public BcastRcver {
	public:
		static ServerRep *GetServerRep(int idx);

	public:
		Client();
		virtual ~Client();

		virtual void configure(const RobotSym *cfg);
		virtual void start();

		void genOid(ServerRep &srvRep, OidRec &oidRec, UniqId &wid);

		virtual void noteXactDone(CltXact *x);

	protected:
		static int GetServerRepIdx(const NetAddr &addr);
		static void FreezeSrvWorkSetCap();

		void configureConnMgrs(const RobotSym *cfg, const SockOpt &opt);
		void scheduleLaunch(Time lastLaunch);
		void launch();
		void genOid2(ServerRep &srvRep, OidRec &oidRec, UniqId &wid);

		virtual void noteInfoEvent(BcastChannel *ch, InfoEvent ev);

	protected:
		static ObjFarm<CltXact> TheXacts;
		static Array<ServerRep*> TheServers; // all servers known to clients
		static int TheShrdWorkSetCap;        // all servers together 

		Array<CltConnMgr*> theConnMgrs;
		RndDistr *theOriginDistr;

		Time theLaunchWin;
		int theLaunchCount; // number of xaction launches

		PopModel *thePopModel; // popularity model to select old oids
		double thePublicRatio; // how often to request a "public" object
		double theRecurrRatio; // how often to re-visit an old object
		bool genUniqUrls;      // each generated URL must be unique

		int thePrivateOid;     // max used oid for private objects
		int theWorkSetCap;     // how far back an oid can refer
};

#endif
