
/*
 * $Id: IOBuf.h,v 1.3 1999/09/07 17:47:01 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef IO_BUF_H
#define IO_BUF_H

#include "LibInit.h"

class IOBuf {
	public:
		IOBuf();
		~IOBuf() { if (theBuf) zip(); }

		void reset() { theOutOff = theInOff = 0; if (theBuf) zip(); }

		bool empty() const { return theOutOff >= theInOff; }
		bool full() const { return theInOff >= theCapacity; }

		char *space() { if (!theBuf) unzip(); return theBuf + theInOff; }
		const char *content() const { if (theBuf) return theBuf + theOutOff; Assert(empty()); return 0; }

		Size spaceSize() const { return theCapacity - theInOff; }
		Size contSize() const { return theInOff - theOutOff; }

		void appended(Size sz) { theInOff += sz; Assert(theInOff <= theCapacity); if (!theBuf) unzip(); }
		void consumed(Size sz) { theOutOff += sz; if (theOutOff == theInOff) reset(); else Assert(theOutOff < theInOff); }
		void consumedAll() { reset(); }
		void pack();

	protected:
		void zip();
		void unzip();

	protected:
		char *theBuf;
		Size theCapacity;
		Size theOutOff;
		Size theInOff;
};


// Do we really need these?
class RdBuf: public IOBuf {
};

class WrBuf: public IOBuf {
	public:
		void append(const char *buf, Size sz);
};


LIB_INITIALIZER(IOBufInit)

#endif
