
/*
 * $Id: LogTagTable.h,v 1.2 1999/09/07 17:47:05 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef LOG_SYMB_TABLE_H
#define LOG_SYMB_TABLE_H

class Log;

// strict order, append-only table
typedef enum { 
	lpNone = 0,

	// special "system" types
	lpLoc,
	lpComment,
	lpSymbTable,
	lpFree = 255,

	// user types
	lpHistogram_,
	lpLineHist,
	lpLog2Hist,
	lpTimeHist,
	lpSizeHist,
	lpTimeVsSizeHist,

	lpStatPing,
	lpStatSoft,

	lpStatPhase,

	// end marker
	lpEnd
} LogEntryType;


// orderless no-delete table
typedef enum { 
	lgNone = 0,

	// special "system" tags
	lgLoc,
	lgSymbTable,
	lgComment,
	lgFree = 255,

	// user tags
	lgStatPing,
	lgStatSoft,

	lgStatPhase_,
	lgStatPhaseWarm,
	lgStatPhaseMeas,
	lgStatPhaseCool,

	// end marker
	lgEnd
} LogEntryTag;


// holds metadata about log entries
class TagDescr {
	public:
		TagDescr(LogEntryTag aTag, LogEntryType aType, const String &aName, int aVer = 0);

		void store(Log &log) const;

	public:
		LogEntryTag theTag;
		LogEntryType theType;
		String theName;
		int theVer;
};

extern void StoreSymbTable(Log &log);

#endif
