
/*
 * $Id: PolyApp.h,v 1.8 1999/09/17 23:54:40 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_APPLICATION_H
#define PG_APPLICATION_H

#include "NetAddr.h"
#include "Array.h"
#include "CmdLine.h"
#include "FileScanner.h"
#include "BcastRcver.h"

class String;
class Agent;
class AgentSym;

class PolyApp: public FileScanTicker, public BcastRcver {
	public:
		PolyApp();
		virtual ~PolyApp();

		virtual void noteMsgStrEvent(BcastChannel *ch, const char *msg);
		virtual void begCycle(int readyCount);
		virtual void endCycle();
		virtual bool tick();

		int run(int argc, char *argv[]);

	protected:
		bool handleCmdLine(int argc, char *argv[]);
		virtual void configure();
		virtual void makeAgents();
		virtual void reportCfg();
		
		void configureLogs(int prec);
		void buildSchedule();
		void parseConfigFile(const String &fname);
		void getIfaces();
		void step();

		virtual void getOpts(Array<OptGrp*> &opts);
		virtual void wasIdle(Time tout);
		virtual Agent *makeAgent(const AgentSym *cfg) = 0;

	protected:
		CmdLine theCmdLine;
		SchArray<in_addr> theIfaces; // list of interface or fake addresses
		Array<const AgentSym*> theLocalCfgs;
		Array<Agent*> theLocals;

		Time theIdleBeg;  // the start of an idle period
		Time theIdleEnd;  // beg + idle_tout
		bool isIdle;

		int theTickCount; // see PolyApp::tick()
};

#endif
