
/*
 * $Id: ContentSym.cc,v 1.3 1999/09/07 17:47:37 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"

#include "String.h"
#include "PglRec.h"
#include "MimeSym.h"
#include "ObjLifeCycleSym.h"
#include "ContentSym.h"


ContentSym::ContentSym(): RecSym("Content", new PglRec) {

	// add fields

	theRec->bAdd("Mime", "mime", new MimeSym);
	theRec->bAdd("size_distr", "size", 0);
	theRec->bAdd("ObjLifeCycle", "obj_life_cycle", new ObjLifeCycleSym);
	theRec->bAdd("float", "cachable", 0);
}

ContentSym::ContentSym(const String &aType, PglRec *aRec): RecSym(aType, aRec) {
	// find fields
}

bool ContentSym::isA(const String &type) const {
	return RecSym::isA(type) || type == "Content";
}

SynSym *ContentSym::clone(const String &type) const {
	if (isA(type))
		return new ContentSym(this->type(), theRec->clone());
	return RecSym::clone(type);
}

MimeSym *ContentSym::mime() const {
	SynSymTblItem *mi = 0;
	Assert(theRec->find("mime", mi));
	Assert(mi->sym());
	return &(MimeSym&)mi->sym()->cast("Mime");
}

RndDistr *ContentSym::size() const {
	return getDistr("size");
}

ObjLifeCycleSym *ContentSym::objLifeCycle() const {
	SynSymTblItem *olci = 0;
	Assert(theRec->find("obj_life_cycle", olci));
	Assert(olci->sym());
	return &(ObjLifeCycleSym&)olci->sym()->cast("ObjLifeCycle");
}

bool ContentSym::cachable(double &ratio) const {
	return getDouble("cachable", ratio);
}
