
/*
 * $Id: PglExprSym.cc,v 1.5 1999/09/07 17:47:43 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"

#include <math.h>

#include "PglExprSym.h"


bool ExpressionSym::isA(const String &type) const {
	return SynSym::isA(type) || type == "Expression";
}

ExpressionSym *ExpressionSym::unOper(const String &op) const {
	cerr << theLoc << "type `" << type() 
		<< "' has no unary operator `" << op << "'" << endl << xexit;
	return 0;
}


ExpressionSym *ExpressionSym::bnOper(const String &op, const SynSym &exp) const {
	cerr << theLoc << "binary operator `" << op 
		<< "' cannot be applied to types `" << type() 
		<< "' and `" << exp.type() << "'" << endl << xexit;
	return 0;
}

void ExpressionSym::checkDenom(int val) const {
	if (val == 0)
		cerr << theLoc << "denominator is zero" << endl << xexit;
}

void ExpressionSym::checkDenom(double val) const {
	if (fabs(val) < 1e-8)
		cerr << theLoc << "denominator is "
			<< (val ? "[virtually]" : "")
			<< " zero" << endl << xexit;
}
