
/*
 * $Id: PglSemx.h,v 1.7 1999/09/17 23:54:56 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef POLYGRAPH_PGL_SEMANTICS_H
#define POLYGRAPH_PGL_SEMANTICS_H

#include "Array.h"
#include "PglArraySym.h"
#include "PglBoolSym.h"
#include "PglExprSym.h"
#include "PglIntSym.h"
#include "PglNetAddrSym.h"
#include "PglQualifSym.h"
#include "PglNumSym.h"
#include "PglRecSym.h"
#include "PglSizeSym.h"
#include "PglStringSym.h"
#include "PglTimeSym.h"
#include "pglSyms.h"

class PglCtx;
class AgentSym;
class DistrSym;
class PhaseSym;
class PopModelSym;

// encapsulates reduce-actions called from parser
// and keeps track of the current parsing context
class PglSemx {
	public:
		static void Init(); // these two are called from parser
		static void Done(); // to initialize/destroy semantics info

		static SynSym *mkCode(StatementSeqSym &);

		static SynSym *mkStatementSeq();
		static SynSym *mkStatementSeq(StatementSym &, StatementSeqSym &);

		static SynSym *mkStatement(SimpleStatementSym &, TokenSym &);
		static SynSym *mkStatement(BlockStatementSym &);

		static SynSym *mkSimpleStatement();
		static SynSym *mkSimpleStatement(SynSym &);

		static SynSym *mkDeclStatement(SynSym &);
		static SynSym *mkExprStatement(SynSym &);
		static SynSym *mkBlockStatement(BlockOpenSym &, SynSym &, BlockCloseSym &);

		static SynSym *mkPureDecl(TypeSym &, CtxNameSym &);
		static SynSym *mkInitDecl(PureDeclSym &, SynSym &, ExpressionSym &);

		static SynSym *mkAssignment(KnownCtxNameSym &, SynSym &, ExpressionSym &);
		static SynSym *mkBlockOpen(TokenSym &);
		static SynSym *mkBlockClose(TokenSym &);

		static SynSym *mkKnownCtxName(CtxNameSym &s);
		static SynSym *mkCtxName(ObjNameSym &);
		static SynSym *mkObjName(TokenSym &);
		static SynSym *mkObjName(TokenSym &, TokenSym &, ObjNameSym &);

		static SynSym *mkExpression(ExpressionSym &);
		static SynSym *mkExpression(ObjNameSym &);
		static SynSym *mkExpression(TokenSym &);
		static SynSym *mkExpression(BlockStatementSym &);
		static SynSym *mkExpression(TypeSym &, BlockStatementSym &);
		static SynSym *mkExpression(TokenSym &, SynSym &s, TokenSym &);
		static SynSym *mkExpression(UnOperSym &op, ExpressionSym &exp);
		static SynSym *mkExpression(ExpressionSym &exp1, TokenSym &op, ExpressionSym &exp2);
		static SynSym *mkExpression(TokenSym &, TokenSym &, ListSym &arr, TokenSym &);

		static SynSym *mkCall(TokenSym &, TokenSym &, ListSym &arr, TokenSym &);

		static SynSym *mkUnOper(TokenSym &);

		static SynSym *mkType(TokenSym &);
		static SynSym *mkType(TokenSym &s, TokenSym &, TokenSym &);

		static SynSym *mkBool(TokenSym &s);
		static SynSym *mkInt(TokenSym &s);
		static SynSym *mkNum(TokenSym &s);
		static SynSym *mkTime(TokenSym &s);
		static SynSym *mkSize(TokenSym &s);
		static SynSym *mkString(TokenSym &s);
		static SynSym *mkAddr(TokenSym &s);
		static SynSym *mkQualif(TokenSym &s);

		static SynSym *mkArray();
		static SynSym *mkArray(ExpressionSym &exp);
		static SynSym *mkArray(ExpressionSym &exp, TokenSym &, ExpressionSym &p);
		static SynSym *mkArray(ArraySym &arr, TokenSym &, ExpressionSym &exp);
		static SynSym *mkArray(ArraySym &arr, TokenSym &, ExpressionSym &exp, TokenSym &, ExpressionSym &p);

		static SynSym *mkList();
		static SynSym *mkList(ExpressionSym &exp);
		static SynSym *mkList(ListSym &arr, TokenSym &, ExpressionSym &exp);

	protected:
		static SynSymTblItem *FindDescr(const TokenSym &name);
		static SynSym &FindSym(const TokenSym &name);
		static SynSymTblItem *Declare(const String &type, const String &name, const TokenLoc &loc);
		static void Assign(SynSymTblItem *i, const ExpressionSym &e);
		static SynSym *Clone(SynSym &, ExpressionSym &);
		static void SetDefault(SynSymTblItem *i);
		static bool KnownType(const String &type);

		static DistrSym *isDistr(const TokenSym &name, ListSym &arr);
		static PopModelSym *isPopModel(const TokenSym &sname, ListSym &args);

		static ArraySym *Place(const AgentSym &srv, const ArraySym &addrs);
		static void Use(const ListSym &agents);
		static void Schedule(const ListSym &phases);

		static SynSym *Any2Any(const SynSym &s, const String &type);
		static IntSym *Any2Int(const SynSym &s);
		static NumSym *Any2Float(const SynSym &s);
		static void NoCast(const SynSym &s, const char *totype);

	public:
		static Array<AgentSym*> TheAgentsToUse; // accumulates placed agents
		static Array<PhaseSym*> TheSchedule;    // scheduled phases
		static Time             TheWorkSetLen;  // working set length
		
	protected:
		static PglCtx *TheCtx;    // current context
		static String TheCtxName; // next context name, if any
};

#endif
