
/*
 * $Id: GLexer.cc,v 1.5 1999/09/07 17:48:15 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "xparser.h"

#include "GLexer.h"

GLexer::GLexer(istream &aSource, const String &aFName):
	theSource(aSource), theFName(aFName) {

	if (!theSource)
		cerr << here << "cannot open `" << theFName << "'" << endl << xexit;
	
	_lookahead.loc = TokenLoc(aFName, 1);
	_lookahead.ch = '\n'; // fool nextCh() to set isFirstOnLine
	nextCh();
}

GLexer::~GLexer() {
}

// interrets the FSA table and performs state transitions
// and returns next token
LexToken GLexer::scan() {
again:
	int state = initialState;
	LexToken token(-1, _lookahead.loc);
	for (;;) {
	    const int next = transition(state, _lookahead.ch);
	    if (next < 0) { // next transition would lead to error state
			token.id(finalInfo[state].tokenValue);
			if (token.id() >= 0) {
				filterProc f = finalInfo[state].filter;
				if (f) 
					f(this, token);
				if (token.id() == 0)
					goto again; // predefined ignored token

				//cerr << here << "token: " << token.spell() << " (" << token.id() << ")" << endl;
				return token;
			} else {
				if (!token.spell().len()) {
					if (_lookahead.ch == EOF) {
						//cerr << here << "token: EOF" << endl;
						return LexToken(1, _lookahead.loc); // predefined EOFTOKEN
					}
					token += _lookahead;
					nextCh();
				}
				//cerr << here << "token: ERR" << endl;
				return LexToken(0, _lookahead.loc); // predefined ERROR
			}
		}
	    token += _lookahead;
	    nextCh();
	    state = next;
	}

	abort();
	return LexToken(-1, _lookahead.loc);
}

void GLexer::nextCh() {
	_lookahead.isFirstOnLine = _lookahead.ch == '\n'; // last char

	char c = theSource.get();
	if (theSource && c == '\n')
		++_lookahead.loc;

	//cerr << here << "next char is `" << c << "' (" << (int)c << ")" << endl;

	_lookahead.ch = theSource ? c : EOF;
}
