
/*
 * $Id: FileScanner.cc,v 1.3 1999/09/07 17:48:25 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "xstd.h"

#include <iostream.h>
#include <iomanip.h>

#include "Assert.h"
#include "FileScanner.h"


/* FileScanUser */

FileScanUser::~FileScanUser() {
}

// A user may not want to implement read or write
// The default implementation will barf in case the
// corresponding method is actually called
void FileScanUser::noteReadReady(int) {
	Assert(false);
}

void FileScanUser::noteWriteReady(int) {
	Assert(false);
}


/* FileScanReserv */

FileScanReserv::FileScanReserv(): theFD(-1), theDir(dirNone), isReady(false) {
}

FileScanReserv::FileScanReserv(int aFD, IODir aDir): theFD(aFD), theDir(aDir), isReady(false) {
}


/* FileScanner */

FileScanner::FileScanner(): theTicker(0), theResCount(0) {
}

FileScanner::~FileScanner() {
}

void FileScanner::configure(int) {
}

void FileScanner::ticker(FileScanTicker *aTicker) {
	Assert(!theTicker);
	theTicker = aTicker;
}

int FileScanner::scan(Time *timeout) {

	if (timeout && *timeout < 0) // no time to sweep
		return 0;

	// silent fix:
	// unfortunately, some selects(2) have magic limits on timeouts
	if (timeout && timeout->tv_sec > 100000000)
		timeout->tv_sec = 100000000;

	//cerr << here << "reserv: " << theResCount << endl;
	const int hotCount = sweep(timeout);
	//cerr << here << "hot files: " << hotCount << endl;

	if (theTicker)
		theTicker->begCycle(hotCount);

	for (int idx = 0; idx < hotCount; ++idx) {
		int fd;
		//cerr << here << "idx: " << idx << " ready: " << readyUser(idx, dirRead, fd) << " & " << readyUser(idx, dirWrite, fd) << endl;
		if (User *u = readyUser(idx, dirRead, fd))
			u->noteReadReady(fd);
		if (User *u = readyUser(idx, dirWrite, fd))
			u->noteWriteReady(fd);
		if (theTicker && !theTicker->tick())
			break;
	}

	if (theTicker)
		theTicker->endCycle();

	return hotCount;
}

FileScanReserv FileScanner::setFD(int fd, IODir dir, FileScanUser *) {
	theResCount++;
	return FileScanReserv(fd, dir);
}

void FileScanner::clearFD(int, IODir) {
	theResCount--;
}
